;******************************************************************************
;    Filename:         pwm615_jc.asm                                          *
;    Author:           Jaroslav Belza                                         *
;******************************************************************************
;    Files Required: P12F615.INC                                              *
;******************************************************************************
; PWM regultor bez displeje.                                                 *
; Kmitoet 500 Hz/8 kHz (GP0 H/L - pin 6 open/GND),                           *
; Regulace kvadratick a linern (GP1 H/L - pin 6 open/GND)                  *
; Vstup pin 3 napt 0 a Vdd, vstup pin 5.                                  *
;                 -----------U----------                                      *
;           +5V --|Vdd              Vss|-- GND                                *
;   nen pouit --|GP5/oscin    GP0/PGD|-- 500 Hz/8 kHz -> open/GND           *
;           Uin --|GP4/oscout   GP1/PGC|-- kvadratick/linern -> open/GND   *
;           Vpp --|GP3/MCLR/Vpp GP2/INT|-- PWM out                            *
;                 ----------------------                                      *
;******************************************************************************
;                                                                             *
;    Revision History:                                                        *
;                                                                             *
;******************************************************************************

     list      p=12f615            ; list directive to define processor
     #include <p12f615.inc>        ; processor specific variable definitions

     __CONFIG   _CP_OFF & _BOR_OFF & _MCLRE_OFF & _WDT_ON & _PWRTE_ON & _INTRC_OSC_NOCLKOUT

		errorlevel -302		; disable warning accessing register not in bank 0
;		errorlevel +302		; re-enable warning accessing register not in bank 0

;***** VARIABLE DEFINITIONS
;w_temp        EQU     0x40        ; variable used for context saving 
;status_temp   EQU     0x41        ; variable used for context saving

tmp		equ	45h
reshi	equ	46h
reslo	equ	47h

	#define	SPEED	GPIO,0		; PWM frequency (0 = 8 kHz/1 = 500 Hz)
	#define	LINQ	GPIO,1		; lineary (0)/quadratic (1) mode

;**********************************************************************
START
	ORG 	0x000			; processor reset vector
; oscillator set
	bsf     STATUS,RP0		; set file register bank to 1 
	movlw	0x00			; set oscillator to factory calibrated frequency
							;nastav osciltor na z vroby kalibrovan kmitoet
	movwf	OSCTUNE
	bcf     STATUS,RP0		; set file register bank to 0
; A/D set
	movlw	0dh				;left justified, Vref=Vdd, AN3, stop, AD on
	movwf	ADCON0
	bsf		STATUS,RP0		;Bank 1
	movlw	58h				;fosc/16, ANS3
	movwf	ANSEL			;switch off ADC inputs on GP0, GP1 a GP2
							;vypne vstupy pro AD, vvody GP0, GP1 a GP2
; I/O set
	movlw	03fh			;Set GP0,1,2,3,4,5 as input (--111111)
							;nastav GP0,1,2,3,4,5 jako vstupy
	movwf	TRISIO			;PWM output will be set as input in TRISIO
							;vstup PWM zatm nastaven jako vstup - je vypnut
	movlw	07fh			;switch on / zapni WPU, WDT = 17 ms
	movwf	OPTION_REG
	movlw	023h			;pull-up GP0, GP1, GP5
	movwf	WPU
	bcf		STATUS,RP0		;Bank 0
	clrf	GPIO
; PWM set
	bsf		STATUS,RP0		;Bank 1
	movlw	0f9h
	movwf	PR2				;perioda 1000 (2 ms/500 Hz or 125 s/8 kHz)
	bcf		STATUS,RP0		;Bank 0
	movlw	00ch
	movwf	CCP1CON			;single output PWM / jeden vstup PWM
	clrf	CCPR1L			;duty-cycle 0% / stda 0%
	bcf 	PIR1,TMR2IF		;clear flag / vynuluj pznak dokonenho cyklu PWM (podle nvodu)
	btfss	SPEED			;speed switch / pepnn rychlosti 0,5/8k
	goto	set8k
set500
	movlw	06h
	movwf	T2CON			;enable TIMER2, prescaler = 16 (500 Hz)
	btfss	PIR1,TMR2IF		;done/hotovo?
	goto	$-1
	bsf		STATUS,RP0		;Bank 1
	bcf 	TRISIO,2		;Set GP2 as output CCP1
	bcf		STATUS,RP0		;Bank 0
loop500
	call	main			;A/D conversion, compute and duty cycle set
	btfsc	SPEED			;speed switch / pepnn rychlosti 0,5/8k
	goto	loop500
	bsf		STATUS,RP0		;Bank 1
	bsf 	TRISIO,2		;Set GP2 as input - CCP1 off
	bcf		STATUS,RP0		;Bank 0
set8k
	movlw	04h
	movwf	T2CON			;enable TIMER2, prescaler = 1 (8 kHz)
	btfss	PIR1,TMR2IF		;done/hotovo?
	goto	$-1
	bsf		STATUS,RP0		;Bank 1
	bcf 	TRISIO,2		;Set GP2 as output CCP1
	bcf		STATUS,RP0		;Bank 0
loop8k
	call	main			;A/D conversion, compute and duty cycle set
	btfss	SPEED			;speed switch / pepnn rychlosti 0,5/8k
	goto	loop8k
	bsf		STATUS,RP0		;Bank 1
	bsf 	TRISIO,2		;Set GP2 as input - CCP1 off
	bcf		STATUS,RP0		;Bank 0
	goto	set500

;----------------podprogramy-----------------

main		;A/D conversion, compute and duty cycle set
	bcf 	PIR1,TMR2IF		;clear flag (synchronizace)

; AD pevod
	bsf 	ADCON0,GO		;start AD pevodu - trv 48 Tcy
	btfsc	ADCON0,GO		;done/hotovo?
	goto	$-1
	clrwdt
	movfw	ADRESH		;

; Linear - Quadratic mode test
	btfsc	LINQ			;lin/q pin test
	goto	quadratic

; Lineary mode
linear
	movwf	CCPR1L			;set MSB bits PWM (duty-cycle = ADRESH)
	movlw	00ch
	movwf	CCP1CON			;clear LSB bits
	goto	testend

quadratic
; Quadratic mode
; AD^2 ->> based on multiplier 8x8 -----------------
; http://www.piclist.com/techref/microchip/math/mul/8x8.htm
; Enter with multiplier in W-Reg, multiplicand in "reslo".
; Result in reshi:reslo.
	movwf	reslo
	clrf	reshi		;
	clrf	tmp 		;
	bsf 	tmp,3 		;
	rrf 	reslo,f		;
loop2	;LOOP:
	btfsc	STATUS,C	;
	addwf	reshi,f		;
	rrf 	reshi,f		;
	rrf 	reslo,f		;
	decfsz	tmp,f		;
	goto	loop2
; multiplication done
	rrf 	reslo,f			;prepare LSB bits
	rrf 	reslo,w			;
	andlw	30h
	iorlw	0ch
	movwf	CCP1CON			;set LSB bits PWM
	movfw	reshi
	movwf	CCPR1L			;set MSB bits PWM (duty-cycle = AD^2)
testend
	btfss	PIR1,TMR2IF		;done/hotovo?
	goto	$-1
	return

END
